@@ -3,11 +3,18 @@
 # Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 """Basic checker for Python code."""
+
+from __future__ import annotations
+
+import argparse
 import collections
 import itertools
 import re
 import sys
-from typing import Dict, Optional, Pattern, Tuple
+from collections.abc import Iterable
+from enum import Enum, auto
+from re import Pattern
+from typing import TYPE_CHECKING, Tuple
 
 import astroid
 from astroid import nodes
@@ -22,26 +29,44 @@ from pylint.checkers.base.name_checker.naming_style import (
     _create_naming_options,
 )
 from pylint.checkers.utils import is_property_deleter, is_property_setter
+from pylint.typing import Options
+
+if TYPE_CHECKING:
+    from pylint.lint.pylinter import PyLinter
+
+_BadNamesTuple = Tuple[nodes.NodeNG, str, str, interfaces.Confidence]
 
 # Default patterns for name types that do not have styles
 DEFAULT_PATTERNS = {
     "typevar": re.compile(
-        r"^_{0,2}(?:[^\W\da-z_]+|(?:[^\W\da-z_]+[^\WA-Z_]+)+T?(?<!Type))(?:_co(?:ntra)?)?$"
+        r"^_{0,2}(?!T[A-Z])(?:[A-Z]+|(?:[A-Z]+[a-z]+)+T?(?<!Type))(?:_co(?:ntra)?)?$"
     )
 }
 
 BUILTIN_PROPERTY = "builtins.property"
-TYPING_TYPE_VAR_QNAME = "typing.TypeVar"
+TYPE_VAR_QNAME = frozenset(
+    (
+        "typing.TypeVar",
+        "typing_extensions.TypeVar",
+    )
+)
 
 
-def _get_properties(config):
+class TypeVarVariance(Enum):
+    invariant = auto()
+    covariant = auto()
+    contravariant = auto()
+    double_variant = auto()
+
+
+def _get_properties(config: argparse.Namespace) -> tuple[set[str], set[str]]:
     """Returns a tuple of property classes and names.
 
     Property classes are fully qualified, such as 'abc.abstractproperty' and
     property names are the actual names, such as 'abstract_property'.
     """
     property_classes = {BUILTIN_PROPERTY}
-    property_names = set()  # Not returning 'property', it has its own check.
+    property_names: set[str] = set()  # Not returning 'property', it has its own check.
     if config is not None:
         property_classes.update(config.property_classes)
         property_names.update(
@@ -50,7 +75,7 @@ def _get_properties(config):
     return property_classes, property_names
 
 
-def _redefines_import(node):
+def _redefines_import(node: nodes.AssignName) -> bool:
     """Detect that the given node (AssignName) is inside an
     exception handler and redefines an import from the tryexcept body.
 
@@ -72,15 +97,15 @@ def _redefines_import(node):
     return False
 
 
-def _determine_function_name_type(node: nodes.FunctionDef, config=None):
+def _determine_function_name_type(
+    node: nodes.FunctionDef, config: argparse.Namespace
+) -> str:
     """Determine the name type whose regex the function's name should match.
 
     :param node: A function node.
     :param config: Configuration from which to pull additional property classes.
-    :type config: :class:`optparse.Values`
 
     :returns: One of ('function', 'method', 'attr')
-    :rtype: str
     """
     property_classes, property_names = _get_properties(config)
     if not node.is_method():
@@ -113,7 +138,9 @@ def _determine_function_name_type(node: nodes.FunctionDef, config=None):
 EXEMPT_NAME_CATEGORIES = {"exempt", "ignore"}
 
 
-def _is_multi_naming_match(match, node_type, confidence):
+def _is_multi_naming_match(
+    match: re.Match[str] | None, node_type: str, confidence: interfaces.Confidence
+) -> bool:
     return (
         match is not None
         and match.lastgroup is not None
@@ -141,7 +168,7 @@ class NameChecker(_BasicChecker):
             },
         ),
         "C0105": (
-            'Type variable "%s" is %s, use "%s" instead',
+            "Type variable name does not reflect variance%s",
             "typevar-name-incorrect-variance",
             "Emitted when a TypeVar name doesn't reflect its type variance. "
             "According to PEP8, it is recommended to add suffixes '_co' and "
@@ -150,15 +177,21 @@ class NameChecker(_BasicChecker):
             "do not require a suffix. The message is also emitted when invariant "
             "variables do have a suffix.",
         ),
-        "W0111": (
-            "Name %s will become a keyword in Python %s",
-            "assign-to-new-keyword",
-            "Used when assignment will become invalid in future "
-            "Python release due to introducing new keyword.",
+        "C0131": (
+            "TypeVar cannot be both covariant and contravariant",
+            "typevar-double-variance",
+            'Emitted when both the "covariant" and "contravariant" '
+            'keyword arguments are set to "True" in a TypeVar.',
+        ),
+        "C0132": (
+            'TypeVar name "%s" does not match assigned variable name "%s"',
+            "typevar-name-mismatch",
+            "Emitted when a TypeVar is assigned to a variable "
+            "that does not match its name argument.",
         ),
     }
 
-    options = (
+    _options: Options = (
         (
             "good-names",
             {
@@ -233,23 +266,21 @@ class NameChecker(_BasicChecker):
                 "These decorators are taken in consideration only for invalid-name.",
             },
         ),
-    ) + _create_naming_options()
+    )
+    options: Options = _options + _create_naming_options()
 
-    KEYWORD_ONSET = {(3, 7): {"async", "await"}}
-
-    def __init__(self, linter):
+    def __init__(self, linter: PyLinter) -> None:
         super().__init__(linter)
-        self._name_category = {}
-        self._name_group = {}
-        self._bad_names = {}
-        self._name_regexps = {}
-        self._name_hints = {}
-        self._good_names_rgxs_compiled = []
-        self._bad_names_rgxs_compiled = []
+        self._name_group: dict[str, str] = {}
+        self._bad_names: dict[str, dict[str, list[_BadNamesTuple]]] = {}
+        self._name_regexps: dict[str, re.Pattern[str]] = {}
+        self._name_hints: dict[str, str] = {}
+        self._good_names_rgxs_compiled: list[re.Pattern[str]] = []
+        self._bad_names_rgxs_compiled: list[re.Pattern[str]] = []
 
-    def open(self):
+    def open(self) -> None:
         self.linter.stats.reset_bad_names()
-        for group in self.config.name_group:
+        for group in self.linter.config.name_group:
             for name_type in group.split(":"):
                 self._name_group[name_type] = f"group_{group}"
 
@@ -257,19 +288,21 @@ class NameChecker(_BasicChecker):
         self._name_regexps = regexps
         self._name_hints = hints
         self._good_names_rgxs_compiled = [
-            re.compile(rgxp) for rgxp in self.config.good_names_rgxs
+            re.compile(rgxp) for rgxp in self.linter.config.good_names_rgxs
         ]
         self._bad_names_rgxs_compiled = [
-            re.compile(rgxp) for rgxp in self.config.bad_names_rgxs
+            re.compile(rgxp) for rgxp in self.linter.config.bad_names_rgxs
         ]
 
-    def _create_naming_rules(self) -> Tuple[Dict[str, Pattern[str]], Dict[str, str]]:
-        regexps: Dict[str, Pattern[str]] = {}
-        hints: Dict[str, str] = {}
+    def _create_naming_rules(self) -> tuple[dict[str, Pattern[str]], dict[str, str]]:
+        regexps: dict[str, Pattern[str]] = {}
+        hints: dict[str, str] = {}
 
         for name_type in KNOWN_NAME_TYPES:
             if name_type in KNOWN_NAME_TYPES_WITH_STYLE:
-                naming_style_name = getattr(self.config, f"{name_type}_naming_style")
+                naming_style_name = getattr(
+                    self.linter.config, f"{name_type}_naming_style"
+                )
                 regexps[name_type] = NAMING_STYLES[naming_style_name].get_regex(
                     name_type
                 )
@@ -278,7 +311,7 @@ class NameChecker(_BasicChecker):
                 regexps[name_type] = DEFAULT_PATTERNS[name_type]
 
             custom_regex_setting_name = f"{name_type}_rgx"
-            custom_regex = getattr(self.config, custom_regex_setting_name, None)
+            custom_regex = getattr(self.linter.config, custom_regex_setting_name, None)
             if custom_regex is not None:
                 regexps[name_type] = custom_regex
 
@@ -289,7 +322,7 @@ class NameChecker(_BasicChecker):
 
         return regexps, hints
 
-    @utils.check_messages("disallowed-name", "invalid-name")
+    @utils.only_required_for_messages("disallowed-name", "invalid-name")
     def visit_module(self, node: nodes.Module) -> None:
         self._check_name("module", node.name.split(".")[-1], node)
         self._bad_names = {}
@@ -298,7 +331,9 @@ class NameChecker(_BasicChecker):
         for all_groups in self._bad_names.values():
             if len(all_groups) < 2:
                 continue
-            groups = collections.defaultdict(list)
+            groups: collections.defaultdict[
+                int, list[list[_BadNamesTuple]]
+            ] = collections.defaultdict(list)
             min_warnings = sys.maxsize
             prevalent_group, _ = max(all_groups.items(), key=lambda item: len(item[1]))
             for group in all_groups.values():
@@ -307,27 +342,29 @@ class NameChecker(_BasicChecker):
             if len(groups[min_warnings]) > 1:
                 by_line = sorted(
                     groups[min_warnings],
-                    key=lambda group: min(warning[0].lineno for warning in group),
+                    key=lambda group: min(  # type: ignore[no-any-return]
+                        warning[0].lineno
+                        for warning in group
+                        if warning[0].lineno is not None
+                    ),
                 )
-                warnings = itertools.chain(*by_line[1:])
+                warnings: Iterable[_BadNamesTuple] = itertools.chain(*by_line[1:])
             else:
                 warnings = groups[min_warnings][0]
             for args in warnings:
                 self._raise_name_warning(prevalent_group, *args)
 
-    @utils.check_messages("disallowed-name", "invalid-name", "assign-to-new-keyword")
+    @utils.only_required_for_messages("disallowed-name", "invalid-name")
     def visit_classdef(self, node: nodes.ClassDef) -> None:
-        self._check_assign_to_new_keyword_violation(node.name, node)
         self._check_name("class", node.name, node)
         for attr, anodes in node.instance_attrs.items():
             if not any(node.instance_attr_ancestors(attr)):
                 self._check_name("attr", attr, anodes[0])
 
-    @utils.check_messages("disallowed-name", "invalid-name", "assign-to-new-keyword")
+    @utils.only_required_for_messages("disallowed-name", "invalid-name")
     def visit_functiondef(self, node: nodes.FunctionDef) -> None:
         # Do not emit any warnings if the method is just an implementation
         # of a base class method.
-        self._check_assign_to_new_keyword_violation(node.name, node)
         confidence = interfaces.HIGH
         if node.is_method():
             if utils.overrides_a_method(node.parent.frame(future=True), node.name):
@@ -339,7 +376,7 @@ class NameChecker(_BasicChecker):
             )
 
         self._check_name(
-            _determine_function_name_type(node, config=self.config),
+            _determine_function_name_type(node, config=self.linter.config),
             node.name,
             node,
             confidence,
@@ -351,20 +388,15 @@ class NameChecker(_BasicChecker):
 
     visit_asyncfunctiondef = visit_functiondef
 
-    @utils.check_messages("disallowed-name", "invalid-name")
-    def visit_global(self, node: nodes.Global) -> None:
-        for name in node.names:
-            self._check_name("const", name, node)
-
-    @utils.check_messages(
+    @utils.only_required_for_messages(
         "disallowed-name",
         "invalid-name",
-        "assign-to-new-keyword",
         "typevar-name-incorrect-variance",
+        "typevar-double-variance",
+        "typevar-name-mismatch",
     )
     def visit_assignname(self, node: nodes.AssignName) -> None:
         """Check module level assigned names."""
-        self._check_assign_to_new_keyword_violation(node.name, node)
         frame = node.frame(future=True)
         assign_type = node.assign_type()
 
@@ -407,6 +439,11 @@ class NameChecker(_BasicChecker):
                     inferred_assign_type, nodes.Const
                 ):
                     self._check_name("const", node.name, node)
+                else:
+                    self._check_name(
+                        "variable", node.name, node, disallowed_check_only=True
+                    )
+
             # Check names defined in AnnAssign nodes
             elif isinstance(
                 assign_type, nodes.AnnAssign
@@ -424,34 +461,29 @@ class NameChecker(_BasicChecker):
         elif isinstance(frame, nodes.ClassDef):
             if not list(frame.local_attr_ancestors(node.name)):
                 for ancestor in frame.ancestors():
-                    if (
-                        ancestor.name == "Enum"
-                        and ancestor.root().name == "enum"
-                        or utils.is_assign_name_annotated_with(node, "Final")
+                    if utils.is_enum(ancestor) or utils.is_assign_name_annotated_with(
+                        node, "Final"
                     ):
                         self._check_name("class_const", node.name, node)
                         break
                 else:
                     self._check_name("class_attribute", node.name, node)
 
-    def _recursive_check_names(self, args):
+    def _recursive_check_names(self, args: list[nodes.AssignName]) -> None:
         """Check names in a possibly recursive list <arg>."""
         for arg in args:
-            if isinstance(arg, nodes.AssignName):
-                self._check_name("argument", arg.name, arg)
-            else:
-                self._recursive_check_names(arg.elts)
+            self._check_name("argument", arg.name, arg)
 
-    def _find_name_group(self, node_type):
+    def _find_name_group(self, node_type: str) -> str:
         return self._name_group.get(node_type, node_type)
 
     def _raise_name_warning(
         self,
-        prevalent_group: Optional[str],
+        prevalent_group: str | None,
         node: nodes.NodeNG,
         node_type: str,
         name: str,
-        confidence,
+        confidence: interfaces.Confidence,
         warning: str = "invalid-name",
     ) -> None:
         type_label = constants.HUMAN_READABLE_TYPES[node_type]
@@ -461,7 +493,7 @@ class NameChecker(_BasicChecker):
             # prevalent group needs to be spelled out to make the message
             # correct.
             hint = f"the `{prevalent_group}` group in the {hint}"
-        if self.config.include_naming_hint:
+        if self.linter.config.include_naming_hint:
             hint += f" ({self._name_regexps[node_type].pattern!r} pattern)"
         args = (
             (type_label.capitalize(), name, hint)
@@ -473,19 +505,26 @@ class NameChecker(_BasicChecker):
         self.linter.stats.increase_bad_name(node_type, 1)
 
     def _name_allowed_by_regex(self, name: str) -> bool:
-        return name in self.config.good_names or any(
+        return name in self.linter.config.good_names or any(
             pattern.match(name) for pattern in self._good_names_rgxs_compiled
         )
 
     def _name_disallowed_by_regex(self, name: str) -> bool:
-        return name in self.config.bad_names or any(
+        return name in self.linter.config.bad_names or any(
             pattern.match(name) for pattern in self._bad_names_rgxs_compiled
         )
 
-    def _check_name(self, node_type, name, node, confidence=interfaces.HIGH):
+    def _check_name(
+        self,
+        node_type: str,
+        name: str,
+        node: nodes.NodeNG,
+        confidence: interfaces.Confidence = interfaces.HIGH,
+        disallowed_check_only: bool = False,
+    ) -> None:
         """Check for a name using the type's regexp."""
 
-        def _should_exempt_from_invalid_name(node):
+        def _should_exempt_from_invalid_name(node: nodes.NodeNG) -> bool:
             if node_type == "variable":
                 inferred = utils.safe_infer(node)
                 if isinstance(inferred, nodes.ClassDef):
@@ -496,7 +535,9 @@ class NameChecker(_BasicChecker):
             return
         if self._name_disallowed_by_regex(name=name):
             self.linter.stats.increase_bad_name(node_type, 1)
-            self.add_message("disallowed-name", node=node, args=name)
+            self.add_message(
+                "disallowed-name", node=node, args=name, confidence=interfaces.HIGH
+            )
             return
         regexp = self._name_regexps[node_type]
         match = regexp.match(name)
@@ -504,87 +545,111 @@ class NameChecker(_BasicChecker):
         if _is_multi_naming_match(match, node_type, confidence):
             name_group = self._find_name_group(node_type)
             bad_name_group = self._bad_names.setdefault(name_group, {})
-            warnings = bad_name_group.setdefault(match.lastgroup, [])
+            # Ignored because this is checked by the if statement
+            warnings = bad_name_group.setdefault(match.lastgroup, [])  # type: ignore[union-attr, arg-type]
             warnings.append((node, node_type, name, confidence))
 
-        if match is None and not _should_exempt_from_invalid_name(node):
+        if (
+            match is None
+            and not disallowed_check_only
+            and not _should_exempt_from_invalid_name(node)
+        ):
             self._raise_name_warning(None, node, node_type, name, confidence)
 
         # Check TypeVar names for variance suffixes
         if node_type == "typevar":
-            self._check_typevar_variance(name, node)
-
-    def _check_assign_to_new_keyword_violation(self, name, node):
-        keyword_first_version = self._name_became_keyword_in_version(
-            name, self.KEYWORD_ONSET
-        )
-        if keyword_first_version is not None:
-            self.add_message(
-                "assign-to-new-keyword",
-                node=node,
-                args=(name, keyword_first_version),
-                confidence=interfaces.HIGH,
-            )
+            self._check_typevar(name, node)
 
     @staticmethod
-    def _name_became_keyword_in_version(name, rules):
-        for version, keywords in rules.items():
-            if name in keywords and sys.version_info < version:
-                return ".".join(str(v) for v in version)
-        return None
-
-    @staticmethod
-    def _assigns_typevar(node: Optional[nodes.NodeNG]) -> bool:
+    def _assigns_typevar(node: nodes.NodeNG | None) -> bool:
         """Check if a node is assigning a TypeVar."""
         if isinstance(node, astroid.Call):
             inferred = utils.safe_infer(node.func)
             if (
                 isinstance(inferred, astroid.ClassDef)
-                and inferred.qname() == TYPING_TYPE_VAR_QNAME
+                and inferred.qname() in TYPE_VAR_QNAME
             ):
                 return True
         return False
 
-    def _check_typevar_variance(self, name: str, node: nodes.AssignName) -> None:
-        """Check if a TypeVar has a variance and if it's included in the name.
-
-        Returns the args for the message to be displayed.
-        """
+    def _check_typevar(self, name: str, node: nodes.AssignName) -> None:
+        """Check for TypeVar lint violations."""
         if isinstance(node.parent, nodes.Assign):
             keywords = node.assign_type().value.keywords
+            args = node.assign_type().value.args
         elif isinstance(node.parent, nodes.Tuple):
             keywords = (
                 node.assign_type().value.elts[node.parent.elts.index(node)].keywords
             )
+            args = node.assign_type().value.elts[node.parent.elts.index(node)].args
 
+        variance = TypeVarVariance.invariant
+        name_arg = None
         for kw in keywords:
-            if kw.arg == "covariant" and kw.value.value:
-                if not name.endswith("_co"):
-                    suggest_name = f"{re.sub('_contra$', '', name)}_co"
-                    self.add_message(
-                        "typevar-name-incorrect-variance",
-                        node=node,
-                        args=(name, "covariant", suggest_name),
-                        confidence=interfaces.INFERENCE,
-                    )
-                return
+            if variance == TypeVarVariance.double_variant:
+                pass
+            elif kw.arg == "covariant" and kw.value.value:
+                variance = (
+                    TypeVarVariance.covariant
+                    if variance != TypeVarVariance.contravariant
+                    else TypeVarVariance.double_variant
+                )
+            elif kw.arg == "contravariant" and kw.value.value:
+                variance = (
+                    TypeVarVariance.contravariant
+                    if variance != TypeVarVariance.covariant
+                    else TypeVarVariance.double_variant
+                )
 
-            if kw.arg == "contravariant" and kw.value.value:
-                if not name.endswith("_contra"):
-                    suggest_name = f"{re.sub('_co$', '', name)}_contra"
-                    self.add_message(
-                        "typevar-name-incorrect-variance",
-                        node=node,
-                        args=(name, "contravariant", suggest_name),
-                        confidence=interfaces.INFERENCE,
-                    )
-                return
+            if kw.arg == "name" and isinstance(kw.value, nodes.Const):
+                name_arg = kw.value.value
 
-        if name.endswith("_co") or name.endswith("_contra"):
+        if name_arg is None and args and isinstance(args[0], nodes.Const):
+            name_arg = args[0].value
+
+        if variance == TypeVarVariance.double_variant:
+            self.add_message(
+                "typevar-double-variance",
+                node=node,
+                confidence=interfaces.INFERENCE,
+            )
+            self.add_message(
+                "typevar-name-incorrect-variance",
+                node=node,
+                args=("",),
+                confidence=interfaces.INFERENCE,
+            )
+        elif variance == TypeVarVariance.covariant and not name.endswith("_co"):
+            suggest_name = f"{re.sub('_contra$', '', name)}_co"
+            self.add_message(
+                "typevar-name-incorrect-variance",
+                node=node,
+                args=(f'. "{name}" is covariant, use "{suggest_name}" instead'),
+                confidence=interfaces.INFERENCE,
+            )
+        elif variance == TypeVarVariance.contravariant and not name.endswith("_contra"):
+            suggest_name = f"{re.sub('_co$', '', name)}_contra"
+            self.add_message(
+                "typevar-name-incorrect-variance",
+                node=node,
+                args=(f'. "{name}" is contravariant, use "{suggest_name}" instead'),
+                confidence=interfaces.INFERENCE,
+            )
+        elif variance == TypeVarVariance.invariant and (
+            name.endswith("_co") or name.endswith("_contra")
+        ):
             suggest_name = re.sub("_contra$|_co$", "", name)
             self.add_message(
                 "typevar-name-incorrect-variance",
                 node=node,
-                args=(name, "invariant", suggest_name),
+                args=(f'. "{name}" is invariant, use "{suggest_name}" instead'),
+                confidence=interfaces.INFERENCE,
+            )
+
+        if name_arg is not None and name_arg != name:
+            self.add_message(
+                "typevar-name-mismatch",
+                node=node,
+                args=(name_arg, name),
                 confidence=interfaces.INFERENCE,
             )
